/*
 * File......: FIELD.PRG
 * Author....: Steve Kolterman [76320,37]
 * Date......: May 22, 1991
 * Revision..: 2.0
 * Log file..: $Logfile$
 * 
 * This is an original work by Steve Kolterman and is placed in the
 * public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log$
 *
 */

#define FGV   fieldget( var )
#define VTFGV valtype( (FGV) )
#define FBV   FieldBlock(var)
#define FBFNV FieldBlock(fieldname(var))
#define EVFBV Eval( (FBV) )
#define VTFV  valtype( EVFBV )
#define VTV   valtype( var )
#define TNUM  IF( VTV=="N",VTFGV,VTFV )
#define TCHR  IF( VTV=="N",FGV,EVFBV )
#define DBS_NAME 1
#translate ATRIM( <x> ) => ALLTRIM( <x> )


#ifdef FT_TEST

#translate Clear() => SCROLL(); SetPos(0,0)
#define NTOC(v) LTRIM(STR( v ))
#define K_ESC 27
#define DEMOCOLOR IF(iscolor(),"+gr/b","+w/n")

  FUNCTION Tester( dbff,numrecs )
     LOCAL oldcolor:= SETCOLOR( DEMOCOLOR ),xx,start,end,key:= 0,;
           fc,o_curs:=SetCursor(0)

     IF (dbff <> NIL) .AND. ( FILE( dbff ) .OR. FILE( dbff+".DBF" ) )
        Clear(); numrecs:= IF( numrecs==NIL,1,VAL(numrecs) )
        USE (dbff); fc:= fcount()

        WHILE numrecs > 0 .AND. key <> K_ESC
           FOR xx:= 1 to fc
              start:= Seconds()
*              ? "Testing SK Field Functions..."
              ? "  DATABASE: ",dbff
              ? "    FIELDS: ",NTOC(fcount())
              ? "    RECORD: ",NTOC(RECNO())
              ? "FIELD NAME: ",fieldname(xx)
              ?
              ? "RETURN values passing a name... "
              ? "  CONTENTS: ",FT_FVal( fieldname(xx) )
              ? "VALUE LENG: ",NTOC( FT_FValLen( fieldname(xx) ) )
              ? "FIELD NUMB: ",NTOC( FT_Fnum( fieldname(xx) ) )
              ? "FIELD TYPE: ",FT_Ftype( fieldname(xx) )
              ? "FIELD LENG: ",NTOC( FT_Flen( fieldname(xx) ) )
              ? "FIELD DECI: ",NTOC( FT_Fdec( fieldname(xx) ) )
              ? "FIELD EXIS: ",FT_Fexist( fieldname(xx) )
              ? "FIELD EMPT: ",FT_Fempty( fieldname(xx) )
              ?
              ? "and...RETURN values passing ordinals"
              ? "  CONTENTS: ",FT_Fval(xx)
              ? "VALUE LENG: ",NTOC( FT_FValLen( (xx) ) )
              ? "FIELD NUMB: ",NTOC( FT_Fnum( (xx) ) )
              ? "FIELD TYPE: ",FT_Ftype( xx )
              ? "FIELD LENG: ",NTOC(FT_Flen( xx ))
              ? "FIELD DECI: ",NTOC(FT_Fdec( xx ))
              ? "FIELD EXIS: ",FT_Fexist( (xx) )
              ? "FIELD EMPT: ",FT_Fempty( (xx) )
              ?
              end:= Seconds()
              ? "Executed In ",TRANSFORM((end -start),"9.999")," Secs."
              ? "Press Any Key; [Esc] To Get Out Now"
              ? key:= INKEY(0); Clear(); IF key==K_ESC; xx:= fc; END
           NEXT
           IF !EOF(); SKIP; ENDIF
           numrecs--
        ENDDO

        CLOSE ALL
        Clear()
     ELSE; Clear()
        Alert( "Bad or No .DBF Parameter",{"Quit"} )
     ENDIF
     SETCOLOR(oldcolor); SetCursor(o_curs)
     QUIT
  RETURN NIL

#endif


/*  $DOC$
 *  $FUNCNAME$
 *     FT_Fval
 *  $ONELINER$
 *     Return the value of a field.
 *  $SYNTAX$
 *      FT_Fval( <xVar> ) -> xVal
 *  $ARGUMENTS$
 *     <xVar> is either a field name or ordinal .DBF position.
 *  $RETURNS$
 *     value (contents) of the specified field.  NIL if error.
 *  $DESCRIPTION$
 *     FT_Fval() reports the value (contents) of any .DBF field.
 *  $EXAMPLES$
 *     xVal:= FT_Fval( "unit_prc" )
 *     xVal:= FT_Fval( 2 )
 *     - or -
 *     nNum:= FT_Fnum( "unit_prc" )
 *     xVal:= FT_Fval( nNum )
 *  $END$
 */

FUNCTION FT_FVal( var )
RETURN IF( FT_FExist(var) , ;
       IF( VTV=="N",FGV,EVFBV ), NIL )


/*  $DOC$
 *  $FUNCNAME$
 *     FT_Ftype
 *  $ONELINER$
 *     Return a field's type, given field name or ordinal position
 *  $SYNTAX$
 *     FT_Ftype( <xVar> ) -> cType
 *  $ARGUMENTS$
 *     <xVar> is either a field name or ordinal .DBF position.
 *  $RETURNS$
 *     the type of field: character (C), numeric (N), date (D), logical (L),
 *     or memo (M).  NIL, if error.
 *  $DESCRIPTION$
 *     FT_Ftype() reports the type ("C","N","D","L","M") of any .DBF field.
 *  $EXAMPLES$
 *     cType:= FT_Ftype( "unit_prc" )
 *     cType:= FT_Ftype( 2 )
 *     - or -
 *     nNum:=  FT_Fnum( "unit_prc" )
 *     cType:= FT_Ftype( nNum )
 *  $END$
 */

FUNCTION FT_FType( var )
RETURN IF( FT_FExist(var) , ;
       IF( VTV=="N",VTFGV,VTFV ), NIL )


/*  $DOC$
 *  $FUNCNAME$
 *     FT_Flen
 *  $ONELINER$
 *     Return a field's length.
 *  $SYNTAX$
 *     FT_Flen( <xVar> ) -> nLen
 *  $ARGUMENTS$
 *     <xVar> is either a field name or ordinal .DBF position.
 *  $RETURNS$
 *     the length of the specified field.  -1 if error.
 *  $DESCRIPTION$
 *     FT_Flen() reports the length of any .DBF field.
 *  $EXAMPLES$
 *     nLen:= FT_Flen("unit_prc")
 *     nLen:= FT_Flen( 2 )
 *     - or -
 *     nNum:= FT_Fnum( "unit_prc" )
 *     nLen:= FT_Flen( nNum )
 *  $END$
 */

FUNCTION FT_FLen( var )
RETURN IF( FT_FExist(var) , ;
       IF( TNUM=="D",len(dtoc( TCHR )),;
       IF( TNUM=="L",1,;
       IF( TNUM=="M",10,;
       IF( TNUM=="C",len( TCHR ),;
       IF( TNUM=="N",len(str( TCHR )), -1 ))))), -1 )


/*  $DOC$
 *  $FUNCNAME$
 *     FT_FValLen
 *  $ONELINER$
 *     Return the length of the value in a field.
 *  $SYNTAX$
 *     FT_Fvallen( <xVar> ) -> nVlen
 *  $ARGUMENTS$
 *      <xVar> is either a field name or ordinal .DBF position.
 *  $RETURNS$
 *      the length of the value in a specified field.  -1 if error.
 *  $DESCRIPTION$
 *      FT_Fvallen() reports the length of the value in any .DBF field.
 *  $EXAMPLES$
 *      nVallen:= FT_Fvallen("unit_prc")
 *      nVallen:= FT_Fvallen( 2 )
 *      - or -
 *      nNum:=    FT_Fnum( "unit_prc" )
 *      nVallen:= FT_Fvallen( nNum )
 *  $END$
 */

FUNCTION FT_FValLen( var )
RETURN IF( FT_FExist(var) , ;
       IF( TNUM=="D",len(dtoc( (TCHR) )),;
       IF( TNUM=="L",1,;
       IF( TNUM=="M",len( ATRIM( TCHR ) ),;
       IF( TNUM=="C",len( ATRIM( TCHR ) ),;
       IF( TNUM=="N",len( ATRIM( str(TCHR) ) ),-1 ))))), -1 )


/*  $DOC$
 *  $FUNCNAME$
 *     FT_FDec
 *  $ONELINER$
 *     Return the number of decimals in a numeric (type "N") field.
 *  $SYNTAX$
 *     FT_Fdec( <xVar> ) -> nDec
 *  $ARGUMENTS$
 *     <xVar> is either a field name or ordinal .DBF position.
 *  $RETURNS$
 *     the number of decimal places in a numeric field.  -1 if field is
 *     not type "N", or other error.
 *  $DESCRIPTION$
 *     FT_Fdec() reports the number of decimal places in a numeric field.
 *  $EXAMPLES$
 *     nDec:= FT_Fdec( "unit_prc" )
 *     nDec:= FT_Fdec( 2 )
 *     - or -
 *     nNum:= FT_Fnum( "unit_prc" )
 *     nDec:= FT_Fdec( nNum )
 *  $END$
 */

FUNCTION FT_FDec( var )
RETURN IF( !FT_FExist(var) .or. TNUM<> "N", -1, ;
       IF( TNUM=="N" .and. "." $str( TCHR ), ;
       len(str( TCHR )) -at(".",str( TCHR )), 0))


/*  $DOC$
 *  $FUNCNAME$
 *     FT_FNum
 *  $ONELINER$
 *     Return the ordinal position given the field name.
 *  $SYNTAX$
 *     FT_Fnum( <xVar> ) -> nNum
 *  $ARGUMENTS$
 *     <xVar> must be a valid field name.
 *  $RETURNS$
 *     the ordinal position of the field.  -1 if error.
 *  $DESCRIPTION$
 *     In 5.01, FT_Fnum() was superseded by FieldPos().  Included here for
 *     those who already coded FT_Fnum() calls.
 *  $EXAMPLES$
 *     nNum:= FT_Fnum( "unit_prc" )
 *  $END$
 */

FUNCTION FT_FNum( var )
RETURN IF( FT_FExist(var) , ;
       IF( VTV=="C", FieldPos(var), var ), -1 )


/*  $DOC$
 *  $FUNCNAME$
 *     FT_FPlace
 *  $ONELINER$
 *     Write a new value to a field.
 *  $SYNTAX$
 *     FT_Fplace( <xVar>,<xVal> ) -> xVal
 *  $ARGUMENTS$
 *     <xVar> is either a field name or ordinal .DBF position.
 *  $RETURNS$
 *     <xVal>, the FT_Fplace()d value.  NIL if error.
 *  $DESCRIPTION$
 *     FT_Fplace() writes a new value to a specified field of *ANY*
 *     Clipper-valid type.  In conjunction with the FIELDPLACE UDC
 *     (in FT_FIELD.CH), it constitutes a fully capable alternative to
 *     REPLACE.
 *  $EXAMPLES$
 *     xVal:= FT_Fplace( "unit_prc",15.73 )
 *     xVal:= FT_Fplace( 2, 15.73 )
 *     - or -
 *     nNum:= FT_Fnum( "unit_prc" )
 *     xVal:= FT_FPlace( nNum,15.73 )
 *  $END$
 */

FUNCTION FT_FPlace( var,value )
RETURN IF( FT_FExist(var),Eval( IF((VTV)=="N",(FBFNV),(FBV)),value ),NIL )


/*  $DOC$
 *  $FUNCNAME$
 *     FT_Fexist
 *  $ONELINER$
 *     Check for the existence of a field.
 *  $SYNTAX$
 *     FT_Fexist( <xVar>,<xVal> ) -> lVal
 *  $ARGUMENTS$
 *     <xVar> may be either a field name or ordinal .DBF position.
 *  $RETURNS$
 *     <lVal>, a logical indicating a field's existence or lack thereof.
 *  $DESCRIPTION$
 *     FT_Fexist() enables existence checking before proceeding with
 *     other operations.
 *  $EXAMPLES$
 *     lExi:= FT_Fexist( "unit_prc" )
 *     lExi:= FT_Fexist( 2 )
 *     - or -
 *     nNum:= FT_Fnum( "unit_prc" )
 *     lExi:= FT_Fexist( nNum )
 *  $END$
 */

FUNCTION FT_Fexist( var )
RETURN IF( (VTV) $ "NC",;
       IF( (VTV)=="N", !Empty( Fieldname(var) ),(FieldPos(var) > 0) ), .F. )


/*  $DOC$
 *  $FUNCNAME$
 *     FT_Fempty
 *  $ONELINER$
 *     Determine if a field is empty.
 *  $SYNTAX$
 *     FT_Fempty( <xVar> ) -> lVal
 *  $ARGUMENTS$
 *     <xVar> may be either a field name or ordinal .DBF position.
 *  $RETURNS$
 *     <lVal>, a logical indicating if a field is empty.
 *  $DESCRIPTION$
 *     FT_Fempty() checks for the existence of a value in a field.
 *  $EXAMPLES$
 *     lEmp:= FT_Fempty( "unit_prc" )
 *     lEmp:= FT_Fempty( 2 )
 *     - or -
 *     nNum:= FT_Fnum( "unit_prc" )
 *     lEmp:= FT_Fempty( nNum )
 *  $END$
 */

FUNCTION FT_Fempty( var )
RETURN IF( FT_FExist(var), (FT_FVallen(var) < 1), .T. )


// EOF: FIELD.prg
